              ; Assembler Flash routines written by Technological Arts and
              ; the following routines were written for early hardware. New
              ; versions of these routines are needed for newer hardware. I
              ; own an older version of hardware, and could not verify operation
              ; of the newer routines. Also space limited my ability to add
              ; routines to determine which type of hardware was present, and
              ; still fit into 4K EEPROM. So they will need to be added by the
              ; user when or if needed. Routine to Erase Flash memory

_EraseFlash:: BSR     _LED_On         ; Indicate Flash Erase active
              clr     _Error          ; Clear Error indication
              LDY     #$9555
              LDAB    #$AA
              LDAA    #1
              STAA    *PPAGE
              STAB    0,Y
              clr     *PPAGE

              movb    #$55,$aaaa
              LDAA    #1
              STAA    *PPAGE
              LDAA    #$80
              STAA    0,Y
              STAB    0,Y
              clr     *PPAGE
              movb    #$55,$aaaa
              LDAA    #1
              STAA    *PPAGE
              LDAA    #$10
              STAA    0,Y
              clr     *PPAGE
              ldy     #FLASH
              LDAB    #$FF
              STAB    0,Y
EF1:          LDAB    0,y             ; msb of Flash byte is low until erased
              BMI     _LED_Off        ; Flash erased ok turn off LED

              LDAB    #$20            ; DQ5=1 if write times out
              BITB    0,y
              BEQ     EF1
              LDAB    0,y             ; check once more
              BMI     _LED_Off        ; Flash erased ok turn off LED

              ; Flashed Failed to erase

              INC     _Error
              RTS

              ; Routine to turn LED On

_LED_On::     LDAA    *PORTT
              ORAA    #$40
              STAA    *PORTT
              RTS

              ; Routine to turn LED Off

_LED_Off::    LDAA    *PORTT
              ANDA    #$B0
              STAA    *PORTT
              RTS

_WriteFlash:: movb    PPAGE,_Curpage  ; preserve PPAGE value
              clr     *PPAGE          ; Flash needs to see address of $5555,
              ; which is in second 16K page of Flash
              inc     *PPAGE          ; so PPAGE=1 for $4000 offset into flash
              movb    #$aa,$9555      ; need offset of $8000 to generate CSP0*
              clr     *PPAGE          ; for Flash CE* then add $1555 ($5555-$4000)
              movb    #$55,$aaaa      ; Flash needs address $2aaa (in first page)
                                      ; so just add $8000 to generate CSP0* for CE*
              inc     *PPAGE          ; same logic as before
              movb    #$a0,$9555
              movb    _Curpage,PPAGE  ; restore PPAGE value
              LDY     _Address
              LDAA    _Data
              staa    0,y             ; write Flash

DataPoll:     ;repeat
              ldaa    _Data           ; get value being programmed
              eora    0,y
              bpl     DPP             ; until DQ7 matches data bit7 or

CheckDQ5:     ldaa    #$20
              bita    0,y
              beq     DataPoll        ; DQ5=1 (timeout failure)

DP_Fail:      ldaa    _Data           ; check one last time before giving up
              eora    0,y
              bpl     _LED_Off
DPP:          ldaa    _Data            ; verify data
              cmpa    0,y
              beq     _LED_Off

              INC     _Error
              BRA     _LED_On

FLASH         = $8000
PORTT         = $AE
PPAGE         = $35


